@@ -75,7 +75,8 @@ class Process(object):
         See create_from, on_save_instance_state and _load_instance_state.
         """
         CLASS_NAME = 'class_name'
-        INPUTS = 'inputs'
+        INPUTS_RAW = 'inputs_raw'
+        INPUTS_PARSED = 'inputs_parsed'
         OUTPUTS = 'outputs'
         PID = 'pid'
         WAITING_ON = 'waiting_on'
@@ -266,10 +267,15 @@ class Process(object):
         bundle[self.BundleKeys.PID.value] = self.pid
 
         # Save inputs
-        inputs = None
+        inputs_raw = None
         if self._raw_inputs is not None:
-            inputs = Bundle(self._raw_inputs)
-        bundle[self.BundleKeys.INPUTS.value] = inputs
+            inputs_raw = Bundle(self._raw_inputs)
+        bundle[self.BundleKeys.INPUTS_RAW.value] = inputs_raw
+
+        inputs_parsed = None
+        if self._parsed_inputs is not None:
+            inputs_parsed = Bundle(self._parsed_inputs)
+        bundle[self.BundleKeys.INPUTS_PARSED.value] = inputs_parsed
 
         bundle[self.BundleKeys.OUTPUTS.value] = Bundle(self._outputs)
 
@@ -342,6 +348,8 @@ class Process(object):
     # Methods that signal events have happened, these should be called by the
     # external processes driving the Process (usually the engine)
     def perform_create(self, pid=None, inputs=None, saved_instance_state=None):
+        if not inputs:
+            inputs = {}
 
         if saved_instance_state is not None:
             self.load_instance_state(saved_instance_state)
@@ -350,20 +358,21 @@ class Process(object):
                 pid = uuid.uuid1()
             self._pid = pid
             self._check_inputs(inputs)
-            if inputs is not None:
-                self._raw_inputs = util.AttributesFrozendict(inputs)
 
-        self._parsed_inputs = \
-            util.AttributesFrozendict(self.create_input_args(self.raw_inputs))
+            self._raw_inputs = util.AttributesFrozendict(inputs)
+            self._parsed_inputs = util.AttributesFrozendict(self.create_input_args(self.raw_inputs))
 
         self._called = False
         self.on_create(pid, inputs, saved_instance_state)
         assert self._called, \
             "on_create was not called\n" \
             "Hint: Did you forget to call the superclass method?"
-        MONITOR.process_created(self)
 
-        self._state = ProcessState.CREATED
+        # Enables fast-forwarding in the on_create method, by explicitly setting
+        # the _state
+        if self._state is None:
+            MONITOR.process_created(self)
+            self._state = ProcessState.CREATED
 
     def perform_start(self):
         """
@@ -566,7 +575,11 @@ class Process(object):
         try:
             return self.fast_forward()
         except error.FastForwardError:
-            return self._run(**self._parsed_inputs)
+            if self._parsed_inputs:
+                kwargs = self._parsed_inputs
+            else:
+                kwargs = {}
+            return self._run(**kwargs)
 
     @protected
     def get_exec_engine(self):
@@ -639,10 +652,14 @@ class Process(object):
     def load_instance_state(self, bundle):
         self._pid = bundle[self.BundleKeys.PID.value]
 
-        inputs = bundle.get(self.BundleKeys.INPUTS.value, None)
+        inputs = bundle.get(self.BundleKeys.INPUTS_RAW.value, None)
         if inputs is not None:
             self._raw_inputs = util.AttributesFrozendict(inputs)
 
+        inputs = bundle.get(self.BundleKeys.INPUTS_PARSED.value, None)
+        if inputs is not None:
+            self._parsed_inputs = util.AttributesFrozendict(inputs)
+
         self._outputs = bundle[self.BundleKeys.OUTPUTS.value].get_dict()
 
         if bundle[self.BundleKeys.WAITING_ON.value]:
